/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module audio.audiomodule;

private import audio.types;
private import audio.audiodata;
private import audio.pin;
private import audio.errors;
private import std.string;
private import std.stdio;
private import util.common;


abstract class AudioModule
{
	private
	{
		char[] m_name;	
		OutputPin[] m_outputs;
		InputPin[] m_inputs;
		ModuleID m_id;
		float m_cyclesStat; // estimation of consumed cycles
	}
		
	protected
	{		
		
		abstract void doProcess();
		
		this(char[] name)
		{
			m_name = name;
			m_outputs.length = 0;
			m_inputs.length = 0;
			m_cyclesStat = 0;
			
		}
	
		final void addInput(char[] name, AudioDataTypeSet expectedType)
		{
			m_inputs.length = m_inputs.length + 1;
			m_inputs[m_inputs.length - 1] = new InputPin(this, name, expectedType);
		}
		
		final void addOutput(char[] name, AudioDataType type)
		{
			m_outputs.length = m_outputs.length + 1;
			m_outputs[m_outputs.length - 1] = new OutputPin(this, name, type);
		}
		
		
		// find the index of a pin with given name
		int findPinIndex(Pin[] arrayOfPins, char[] pinName)
		{
			for(int i = 0; i < arrayOfPins.length; i++)
			{
				if (arrayOfPins[i].name == pinName) 
				{
					return i;	
				}
			}	
			throw new NoSuchPinError(name, pinName);
		}
		
		Pin findPin(Pin[] arrayOfPins, char[] pinName)
		{
			return arrayOfPins[findPinIndex(arrayOfPins, pinName)];
		}
		
	}		
	
	public
	{
		
		char[] name()
		{
			return m_name;	
		}
		
		InputPin input(char[] name)
		{
			return cast(InputPin) findPin(m_inputs, name);
		}
		
		OutputPin output(char[] name)
		{
			return cast(OutputPin) findPin(m_outputs, name);
		}
		
		InputPin input(int i)
		{
			return m_inputs[i];
		}
		
		OutputPin output(int i = 0)
		{
			return m_outputs[i];
		}
		
		void updateTo(AudioCycle targetCycle )
		{
			for (int i = 0; i < m_inputs.length; i++)
			{
				m_inputs[i].updateTo(targetCycle);
			}
			
			debug(2)writefln(format("    %s::doProcess::debut", m_name));	
			final long startCycles = rdtsc();
			doProcess();
			final long stopCycles = rdtsc();
			final long cycles = stopCycles - startCycles;
			
			m_cyclesStat = m_cyclesStat * 0.99 + cycles * 0.01;
			
			
			debug(2) writefln(format("    %s::doProcess::fin, took %s cycles", m_name, stopCycles - startCycles));	
			for (int i = 0; i < m_outputs.length; i++)
			{
				m_outputs[i].setLastCycle(targetCycle);
			}			
		}
		
		
		InputPin[] inputs()
		{
			return m_inputs;
		}
		
		OutputPin[] outputs()
		{
			return m_outputs;
		}
				
		ModuleID id(ModuleID newId)
		{
			m_id = newId;
			return newId;
		}
		
		ModuleID id()
		{
			return m_id;	
		}
		
		float cycles()
		{
			return m_cyclesStat;	
		}
	}	
	
}