/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module audio.audiomodule;

private import audio.types;
private import audio.audiodata;
private import audio.modules.smoother;

import std.stdio;



class BadInputType : object.Exception
{	
	this(char[] msg)
	{
		super(msg);	
	}
}


abstract class AudioModule
{
	private
	{
		AudioData[] output;		
		AudioData[] input;			
	}
		
	protected
	{
		abstract void doProcess();
	
		void addInput(AudioData audioData)
		{
			input.length = input.length + 1;
			input[input.length - 1] = audioData;					
		}
		
		void addOutput(AudioDataType type)
		{
			output.length = output.length + 1;
			output[output.length - 1] = AudioDataFactory.create(type, this);		
		}
		
		void setInput(int i, AudioData audioData, AudioDataTypeSet expectedTypes)
		{
			AudioDataType type = audioData.getType();
			if (type & expectedTypes)
			{
				input[i] = audioData;	
			} else
			{
				// the input is not accepted, but we try to "box" it with some convenient modules
				
				// FLOAT => ANALOG 
				// boxing the input into a Smoother	
				if ((type == AudioDataType.FLOAT) && ((expectedTypes & AudioDataType.ANALOG) != 0))
				{
					auto smoother = new Smoother(audioData);
					input[i] = smoother.getOutput(0);
				} else
				{
					throw new BadInputType("Bad input type.");
				}
			}			
		}
			
	}		
	
	public
	{
		this()
		{
			// no output, no input
			output.length = 0;
			input.length = 0;
		}	
		
		void updateTo(int targetCycle)
		{
			for (int i = 0; i < input.length; i++)
			{
				assert(input[i] !is null);
				input[i].updateTo(targetCycle);
			}		
			doProcess();
			for (int i = 0; i < output.length; i++)
			{
				output[i].setLastCycle(targetCycle);
			}		
		}
		
		int getNumInputs()
		{
			return input.length;
		}
		
		int getNumOutputs()
		{
			return output.length;	
		}
		
		AudioData getOutput(int i)
		{
			return output[i];
		}
		
		AudioData getInput(int i)
		{
			return input[i];
		}
		
		
		
	}	
	
}