/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.balleffect;


import demo.effect;
import demo.demo;
import util.all;
import util.geometry.all;
import gl.all;


final class BallEffect : Effect
{
	private 
	{
		Ball[] m_ball;
		ShaderProgram m_shader;
		
		vec4f[] m_colors; // color to be transmitted to GPU
		vec3f[] m_pos; // position to be transmitted to GPU
		vec3f[] m_mov; // movement to be transmitted to GPU
		float[] m_intensity;
	}
	
	public
	{
		this(vec2d interval, int priority)
		{
			super("BallEffect", interval, priority);			
		}					
		
		override bool isPaperEffect() { return false; }
			
		override void onLoad() // load resources
		{
			m_ball.length = m_colors.length = m_pos.length = m_mov.length = m_intensity.length = 8;
			
			for (int i = 0; i < m_ball.length; i++)
			{
				m_ball[i].pos = vec3f(randGaussf(),randGaussf(),randGaussf());	
				m_ball[i].mov = vec3f(randGaussf(),randGaussf(),randGaussf()) * 0.5;
				m_ball[i].intensity = randFloat();	
				
				Colorf c = hsla_to_rgba(i / cast(float)m_ball.length, 0.4f, 0.5, 1.f);				
				//if (i == 0) c = -c;
				m_ball[i].color = vec4f(c.x, c.y, c.z, c.w);
			}
			
			m_shader = new ShaderProgram("data/shaders/ball_vertex.glsl", "data/shaders/ball_pixel.glsl");
		}
		
		override void onDraw() // draw effect
		{
			// transmit to graphics
			demo.mainFBO.use();
			//FBO.getDefault().use();
			
			// define camera
			auto modelView = mat4f.lookAt(vec3f(0,0,-2), vec3f(0,0,0), vec3f(0.f,1.f,0.f));
		    auto projection = mat4f.perspective( 50, 1.0, 0.1, 200.0);
		    
		    auto totalMatrix = modelView; //projection * modelView;
		    		    
			for (int i = 0; i < m_ball.length; i++)
			{
			 	m_colors[i] = m_ball[i].color; // color to be transmitted to GPU
			 	auto tr = totalMatrix * vec4f(m_ball[i].pos, 1.0);
			 	m_pos[i] = tr.xyz / tr.w;
			 	m_mov[i] = m_ball[i].mov;
			 	m_intensity[i] = m_ball[i].intensity;
			}
			
			m_shader.use();
		    m_shader.set4fv("col", m_colors);
		    m_shader.set3fv("pos", m_pos);
		    m_shader.set1fv("intensity", m_intensity);
		    m_shader.set1f("localTime", localTime);
		    m_shader.set1f("progression", progression);		    
		    
		    
		    //m_shader.setMat4f("modelview", modelView);
		    //m_shader.setMat4f("projection", projection);
			
			GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f)); // to span all the screen
			GL.modelviewMatrix = mat4f.IDENTITY;
			
			
			GL.enable(GL_TEXTURE_2D);
			GL.disable( [ GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH, GL_BLEND ] );
			
			
			
			float ratio = demo.ratio;
			glBegin(GL_QUADS);
				glTexCoord2f(-ratio,1); glVertex2f(-ratio,-1);	
				glTexCoord2f(ratio,1); glVertex2f(+ratio,-1);
				glTexCoord2f(ratio,-1); glVertex2f(+ratio,+1);
				glTexCoord2f(-ratio,-1); glVertex2f(-ratio,+1);				
			glEnd();	
		}
		
		void onMove(double dt) // move effect
		{
			for (int i = 0; i < m_ball.length; i++)
			{
				m_ball[i].move(dt, demo.width, demo.height);	
			}		
		}
	}	
}


private struct Ball
{
	public
	{
		vec3f pos;
		vec3f mov;		
		vec4f color;	
		float intensity;
		
		void move(float dt, int width, int height)
		{
			mov += vec3f(randGaussf(), randGaussf(), randGaussf()) * 0.5f * dt;
			
			mov -= pos * 0.5f * dt;
			
			pos += mov * dt * 1.0;
		}
	}
}

