/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.bitmapfont;

import gl.all;
import util.all;
import util.geometry.all;

final class BitmapFont
{
	private
	{
		Texture2D m_tex; // the letters
		Texture2D m_fillTex; // filling
		ShaderProgram m_shader;
		int m_widths[256];		// width of each character
		
		
		
		void drawChar(float x, float y, float factor, char a, vec4f color1, vec4f color2)
		{
			float cw = cellWidth;
			float ch = cellHeight;
			auto xmin = x ;
			auto xmax = x + cw * factor;
			auto ymin = y ;
			auto ymax = y + ch * factor;
			
			auto prout = 0.005f;
			auto xmin2 = x - prout ;
			auto xmax2 = x + cw * factor + prout;
			auto ymin2 = y - prout;
			auto ymax2 = y + ch * factor + prout;
			
			
			ubyte b = cast(ubyte) a;
			
			int ligne = (b / 16);
			int col = (b & 15);
			
		    float s = col * (1.0f / 16.0f) + 0.01f / 16.f;
		    float s2 = s + (0.98f / 16.0f);
		    
		    float t = ligne * (1.0f / 16.0f) + 0.01f / 16.f;
		    float t2 = t + (0.98f / 16.0f);		
		    
		    glColor4f(0.f, 0.f, 0.f, 1.f); 
		    glTexCoord2f(s, t);	glVertex2f(xmin2, ymax2);
			glTexCoord2f(s2, t);  glVertex2f(xmax2, ymax2);
			glTexCoord2f(s2, t2); glVertex2f(xmax2, ymin2);
			glTexCoord2f(s, t2);  glVertex2f(xmin2, ymin2);    
		    
	    	glColor4fv(color1.ptr); glTexCoord2f(s, t);	glVertex2f(xmin, ymax);
			glColor4fv(color2.ptr); glTexCoord2f(s2, t);  glVertex2f(xmax, ymax);
			glColor4fv(color2.ptr); glTexCoord2f(s2, t2); glVertex2f(xmax, ymin);
			glColor4fv(color1.ptr); glTexCoord2f(s, t2);  glVertex2f(xmin, ymin);
			
			
			
		}
	}	
	
	public
	{
		this(char[] file, char[] file2)
		{
			
			{
				// load text texture
				m_tex = new Texture2D(Texture.IFormat.RGBA8);
				m_tex.minFilter = Texture.Filter.LINEAR_MIPMAP_LINEAR;
				m_tex.magFilter = Texture.Filter.LINEAR;
				m_tex.wrapS = Texture.Wrap.CLAMP_TO_EDGE;
				m_tex.wrapT = Texture.Wrap.CLAMP_TO_EDGE;
				scope DevilImage img = new DevilImage(file);
				m_tex.setImage(0, img.width, img.height, Texture.Format.RGBA, Texture.Type.UBYTE, img.data, true);
				
				// load filling texture
				m_fillTex = new Texture2D(Texture.IFormat.RGBA8);
				m_fillTex.minFilter = Texture.Filter.LINEAR_MIPMAP_LINEAR;
				m_fillTex.magFilter = Texture.Filter.LINEAR;
				m_fillTex.wrapS = Texture.Wrap.REPEAT;
				m_fillTex.wrapT = Texture.Wrap.REPEAT;
				scope DevilImage img2 = new DevilImage(file2);
				m_fillTex.setImage(0, img2.width, img2.height, Texture.Format.RGBA, Texture.Type.UBYTE, img2.data, true);
				
				
				// analysis to find the width of each character
				auto scope image = new Image(img);
				
				for (int i = 0; i < 256; ++i)
				{
					int ligne = (i / 16);
					int col = (i & 15);
					
					int bx = cellWidth() * col;
					int by = cellHeight() * ligne;
					
					m_widths[i] = cellWidth() >> 1;
					
					for (int x = cellWidth() - 1; x >= 0; --x)
					{
						for (int y = 0; y < cellHeight(); ++y)
						{
							uint c = image.getPixel(bx + x, by + y);
							
							if (Avalue(c) > 0)
							{
								m_widths[i] = x;
								goto end_of_loops;
							}
						}
					}
					
					end_of_loops: ;
					
				}
				
			}
			
			m_shader = new ShaderProgram("data/shaders/font_vertex.glsl", "data/shaders/font_pixel.glsl");
			m_shader.setSampler("tex", m_tex, 0);
			m_shader.setSampler("fill", m_fillTex, 1);
			
		}		
		
		int cellWidth()
		{
			return m_tex.width / 16;	
		}
		
		int cellHeight()
		{
			return m_tex.height / 16;	
		}
		
		int charWidth(char a)
		{
			ubyte b = cast(ubyte) a;
			return m_widths[b];
		}
		
		int charHeight()
		{
			return cellHeight();
		}
		
		// draw a string on the screen, centered in (x, y) with a given expected width 
		void drawString(float x, float y, float width, vec4f color1, vec4f color2,  char[] s)
		{
			if (s == "") return;
			
			GL.disable( [ GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH] ); 		    
		    GL.enable(GL_BLEND);
		    GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA,
 			            GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);
			
 			            
		    m_shader.use();
		    
		    
		    float real_width = getStringWidth(s);
		    float factor = width / real_width;
		    
		    float bx = x - width * 0.5f;		    
		    
		    glBegin(GL_QUADS);								 
				for (int i = 0; i < s.length; ++i)
				{
					float t = i / cast(float)s.length;
					float t2 = (i+1) / cast(float)s.length;
					vec4f c1 = (1.f - t) * color1 + t * color2;
					vec4f c2 = (1.f - t2) * color1 + t2 * color2;
					
					drawChar( bx, y - factor * (charHeight() * 0.5f), factor, s[i], c1, c2 );
					bx += charWidth(s[i]) * factor;
				}
				
			glEnd();
		}
		
		int getStringWidth(char[] s)
		{
			int w = 0;
			for (int i = 0; i < s.length; ++i)
			{
				w += charWidth(s[i]) + 2;	
			}
			return w;
		}
		
		int getStringHeight(char[] s)
		{
			return charHeight();
		}
		
	}
}