/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.effect;

private import util.math;
private import util.all;
private import util.geometry.all;
private import gl.all;
private import demo.demo;

/**
 * An Effect is a flexible demo effect.
 *
 * At each frame, a set of effects are "drawed" sequentially, in the full viewport, to the main Texture
 * the onDraw virtual function can do anything relative to openGL and such.
 *
 * Context informations are passed through a Demo object which contains all effects
 *
 * NB : if an effect has to be multipass it has to use textures and framebuffer itself
 */ 
class Effect
{
	private
	{
		vec2d m_interval; // [startime, stoptime]
		int m_priority; // the lower, the sooner redrawn
		bool m_loaded;
		char[] m_name;
		Demo m_demo;					
		double m_localTime, m_progression, m_globalTime;
	}
	
	public
	{
		this(char[] name, vec2d interval, int priority)
		{
			m_demo = null;
			m_name = name;
			m_interval = interval;
			assert(interval.x < interval.y);
			m_priority = priority;
			m_loaded = false;
			m_demo = null;
		}
		
		final Demo demo()
		{
			return m_demo;	
		}
		
		final Demo demo(Demo d)
		{
			return m_demo = d;	
		}
		
		final char[] name()
		{
			return m_name;	
		}
		
		final bool isActive(double globalTime)
		{			
			return (startTime <= globalTime) && (globalTime < endTime);
		}
		
		final double startTime()
		{
			return m_interval.x;	
		}
		
		final float endTime()
		{
			return m_interval.y;	
		}
		
		final double duration()
		{
			return endTime - startTime;	
		}
		
		final float ratio()
		{
			return m_demo.ratio;	
		}
		
		final int priority()
		{
			return priority;	
		}
		
		final int priority(int p)
		{
			return (m_priority = p);	
		}
		
		final double progression()
		{
			return m_progression;	
		}
		
		final double localTime()
		{
			return m_localTime;
		}
		
		final double globalTime()
		{
			return m_globalTime;
		}
		
		float BPM()
		{
			return m_demo.BPM();
		}
		
		float timePerBeat()
		{
			return m_demo.timePerBeat();
		}
		
		int globalBeat()
		{
			return trunc(globalTime() / timePerBeat());
		}
		
		int localBeat()
		{
			return trunc(localTime() / timePerBeat());
		}
		
		final void load()
		{
			if (!m_loaded)
			{
				m_loaded = true;
				onLoad();
			}	
		}
		
		final void draw(double globalTime)
		{
			load(); // ensure the effect is loaded
			m_globalTime = globalTime;
			m_localTime = globalTime - startTime;
			m_progression = m_localTime / duration;
			assert(m_progression >= 0.0);
			assert(m_progression < 1.0);
			
		//	debug(1) writefln("draw : %s", name);
			onDraw();			
		}
		
		final void move(double globalTime, double dt)
		{
			load(); // ensure the effect is loaded
			m_globalTime = globalTime;
			m_localTime = globalTime - startTime;
			m_progression = m_localTime / duration;
			assert(m_progression >= 0.0);
			assert(m_progression < 1.0);
		//	debug(1) writefln("move : %s", name);
			onMove(dt);			
		}
		
		abstract void onLoad(); // load resources
		abstract void onDraw(); // draw effect on a texture
		abstract void onMove(double dt); // move effect
		abstract bool isPaperEffect(); // does it need paper rendering ?
		
		int opCmp(Object o) // for sorting effects by priority
   		{
			auto e = cast(Effect) o;
        	if (!e) 
        	{
	        	return -1;
        	} else 
        	{
	        	return m_priority - e.m_priority;
        	}
   		}
   		
   		
   		
	}
}

