/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.gdologo;

import gl.all;
import util.all;
import util.geometry.all;
import std.stream;
import std.regexp;
import std.stdio;
import std.string;
import std.conv;

class GDOException : object.Exception
{	
	this(char[] filename, ulong linenumber, char[] msg)
	{
		super(format("%s(%s) : %s", filename, linenumber, msg));
	}
}

// to open Graindolium's files
// GDO files are open and separated in multiple small lines
class GDOLogo
{
	
	private
	{
		
		class GDOLine 
		{
			public
			{
				vec4f m_aColor, m_bColor;
				vec2f m_a, m_b;
				float m_width;
				float m_length;
				vec2f m_bma;
				
				void draw(float width, vec4f color)
				{					
					float w2 = width * 0.5f;
					vec2f c = w2 * vec2f(-m_bma.y, m_bma.x);
					
					glColor4fv((m_aColor * color).ptr);					
					GL.vertex(m_a + c, m_a - c);
					glColor4fv((m_bColor * color).ptr);					
					GL.vertex(m_b - c, m_b + c);
				}
				
				void draw2(float width, vec4f color)
				{					
				//	glColor4fv(m_aColor.ptr);
					
				//	glColor4fv(m_bColor.ptr);
					width *= 0.5f;
					const N = 4;
					
					static vec2f[N+1] vv;
					static bool vv_inited = false;
					
					if (!vv_inited)
					{
						vv_inited = true;
						for (int i = 0; i <= N; i++)
						{
							auto t = TWO_PI_F * i / cast(float)N;
							vv[i] = vec2f(cos(t), sin(t));
						}
					}
					
					glColor4fv((m_aColor * color).ptr);	
					for (int i = 0; i < N; i++)
					{
						GL.vertex(m_a, m_a + vv[i] * width, m_a + vv[i+1] * width);						
					}
					
					glColor4fv((m_bColor * color).ptr);	
					for (int i = 0; i < N; i++)
					{
						GL.vertex(m_b, m_b + vv[i] * width, m_a + vv[i+1] * width);						
					}
				}
				
				
				this(vec2f a, vec4f a_color, vec2f b, vec4f b_color, float width)
				{
			//		assert((a.x != b.x) || (a.y != b.y));
					m_a = a;
					m_b = b;
					m_aColor = a_color;
					m_bColor = b_color;
					m_width = width;
					m_length = 0.f;
					
					m_bma = (b - a).normalized;
				}	
				
				float length()
				{
					return m_length;
				}
				
				void computeLength()
				{
					m_length = m_a.distanceTo(m_b);
				}
				
				// the angle
				float angle()
				{
					return atan2(m_b.y - m_a.y, m_b.x - m_a.x);
				}
				
				// linear interp
				vec2f getPoint(float t)
				{
					return m_a + (m_b - m_a) * t;
				}
				
				// color linear interp
				vec4f getColor(float t)
				{
					return m_aColor + (m_bColor - m_aColor) * t;
				}
			}
		}
		
		Vector!(GDOLine) m_lines;
		float m_lengthOfLines; // total length (to pick random point)
		
		
			
		
		void addLine(GDOLine line)
		{
			line.computeLength();
			
			auto const SUBDIV = 10.f; // no created GDOLine will be greater than 5 in length (not normalized)
		
			
			// subdivise line in several lines
			
			int NEEDED = 1 + round(line.length() / SUBDIV);
			
			for (int i = 0; i < NEEDED; ++i)
			{
				vec2f a = line.getPoint(i / cast(float)NEEDED); 
				vec2f b = line.getPoint((i+1) / cast(float)NEEDED);
				//auto ca = line.getColor(i / cast(float)NEEDED); 
				//auto cb = line.getColor((i+1) / cast(float)NEEDED); 
				// THIS IS A HACK
				auto ca = hsla_to_rgba(randFloat, randFloat, randFloat, 1.0);
				auto cb = hsla_to_rgba(randFloat, randFloat, randFloat, 1.0);
				m_lines.add(new GDOLine(a, ca, b, cb, line.m_width));
			}
		}
		
		float secure_div(float a, float b)
		{
			const float A_SMALL_FLOAT = 1e-10f;
			if (abs(b) < A_SMALL_FLOAT) 
			{
				if (b >= 0) b = A_SMALL_FLOAT;
				else b = -A_SMALL_FLOAT;
			}
			return a / b;
		}
		
		// generic 3-point lagrande interpolation
		T lagrangeInterp3(T)(float x, float x1, T y1, float x2, T y2, float x3, T y3)
		{
			float f1 = ((x - x2) * (x - x3)) / ((x1 - x2) * (x1 - x3));
			float f2 = ((x - x1) * (x - x3)) / ((x2 - x1) * (x2 - x3));
			float f3 = ((x - x1) * (x - x2)) / ((x3 - x2) * (x3 - x1));
			return f1 * y1 + f2 * y2 + f3 * y3;
		}
		
		void addCurve(float xxl1, float yyl1, float xxl2, float yyl2, float xxl3, float yyl3, vec4f currentColor, float currentWidth)
		{
			auto const SUBDIV = 10.f; // no created GDOLine will be greater than 5 in length (not normalized)
		
			auto la = new GDOLine(vec2f(xxl1, yyl1), currentColor, vec2f(xxl2, yyl2), currentColor, currentWidth);
			auto lb = new GDOLine(vec2f(xxl2, yyl2), currentColor, vec2f(xxl3, yyl3), currentColor, currentWidth);
			
	//		m_lines.add(la);
	//		m_lines.add(lb);
			
			la.computeLength();
			lb.computeLength();
			auto estimated_length = la.length() + lb.length();
			if (abs(estimated_length) < SUBDIV ) return;
			
			int NEEDED = 1 + round(estimated_length / SUBDIV); // number of sudivision			
		
			vec2f previous = vec2f(xxl1, yyl1);
			vec2f A = vec2f(xxl1, yyl1);
			vec2f B = vec2f(xxl2, yyl2);
			vec2f C = vec2f(xxl3, yyl3);
			
			for (int uj = 1; uj <= NEEDED; ++uj)
			{
				vec2f current = lagrangeInterp3(uj / cast(float)NEEDED, 0.f, A, 0.5f, B, 1.f, C);
				m_lines.add(new GDOLine(previous, currentColor, current, currentColor, currentWidth));
				previous = current;
			}
			
		}
		
		
		class Token
		{
			public
			{
				char[] s;	
				ulong line;
				
				this(char[] _s, ulong _line)
				{
					s = _s;
					line = _line;
				}
			}	
		}
		
		static ShaderProgram m_shader = null;
		
	}
	
	public
	{
		this()
		{
			m_lines = new Vector!(GDOLine)();
			
			if (m_shader is null)
			{
				m_shader = new ShaderProgram("data/shaders/put_vertex.glsl", "data/shaders/put_pixel.glsl");
			}
		}
		
		this(char[] filename)
		{
			this();
			load(filename);			
		}
		
		Vector!(GDOLine) lines()
		{
			return m_lines;
		}
		
		void load(char[] filename)
		{
			if (filename is null) return;	
			
			
			auto tokens = new Vector!(Token)(); 
			
			{
				auto scope file = new BufferedFile(filename);
				int nol = 0;			
				
				while (!file.eof()) 
				{
					char[] line = file.readLine();	
					++nol;
				
					foreach (s; std.regexp.split(line, "\\s+"))
	 				{
		 			
		 				// eliminate empty token
		 				if (s != "") tokens.add(new Token(s, nol));
	 				}
				}
				file.close();
			}
			
			bool isComment(Token token)
			{
				return token.s[0] == '#';
			}			
			
			int currentToken = 0;
			
			Token nextToken()
			{
				while(currentToken < tokens.size)
				{
					Token t = tokens.get(currentToken);
					
					currentToken++;
					
					if (isComment(t)) // remove comments
					{
						while(currentToken < tokens.size)
						{
							Token t2 = tokens.get(currentToken);	
							if (t2.line != t.line) break;
							else currentToken++;
						} 
						
					} else return t;
				}	
				return null;
			}
			
					
			
			// interpreting the GDO
			
			Token t = nextToken();
			if (t.s != "GDO")
			{
				throw new GDOException(filename, t.line, format("not a GDO file (%s instead)", t.s));
			}
			
			t = nextToken();
			if (t.s != "version")
			{
				throw new GDOException(filename, t.line, "'version' directive expected");				
			}
			
			t = nextToken();
			int ver = toInt(t.s);
			
			if (ver < 0) throw new GDOException(filename, t.line, "such a version does not exist. Don't cheat with me");				
			if (ver > 1) throw new GDOException(filename, t.line, "unsupported version");
			
			vec4f currentColor = vec4f(1.f,1.f,1.f,1.f);
			float currentWidth = 3.f;
			
			Token tok;
			while (( tok = nextToken()) !is null)
			{
				switch(tok.s)
				{
					case "curve3":
					
					// subdivise curve
						float xxl1 = atof(nextToken().s);
						float yyl1 = atof(nextToken().s);
						float xxl2 = atof(nextToken().s);
						float yyl2 = atof(nextToken().s);
						float xxl3 = atof(nextToken().s);
						float yyl3 = atof(nextToken().s);
						addCurve(xxl1, yyl1, xxl2, yyl2, xxl3, yyl3, currentColor, currentWidth);						
						break;
						
					case "line":
						float x1 = atof(nextToken().s);
						float y1 = atof(nextToken().s);
						float x2 = atof(nextToken().s);
						float y2 = atof(nextToken().s);
						addLine(new GDOLine(vec2f(x1, y1), currentColor, vec2f(x2, y2), currentColor, currentWidth));
						break;
						
					case "color":
						float r = atof(nextToken().s) / 255.f;
						float g = atof(nextToken().s) / 255.f;
						float b = atof(nextToken().s) / 255.f;
						currentColor = vec4f(r, g, b, 1.f);
						break;
						
					case "size":
						currentWidth = atof(nextToken().s);
						break;
						
					
					default:
						throw new GDOException(filename, tok.line, format("unknown token : %s", tok.s ));
				}
			}			
			normalize();
			findtotalLength();
			
		}
		
		void draw(mat4f transfo, vec4f color, float width, bool add)
		{
			m_shader.use();
			GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, add ? GL.BlendFactor.ONE : GL.BlendFactor.ONE_MINUS_SRC_ALPHA, 
		                GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);	
			
			glPushMatrix();
			
			glBegin(GL_TRIANGLES);
			glColor4fv(color.ptr);
			foreach(GDOLine l; m_lines) 
			{
				l.draw2(width, color);
			}
			glEnd();			
			
			glBegin(GL_QUADS);
			glColor4fv(color.ptr);
			foreach(GDOLine l; m_lines) 
			{
				l.draw(width, color);
			}
			glEnd();
			
			glPopMatrix();
		}
		
		// a random line on the GDO
		GDOLine randLine()
		{/*
			
			float d = randFloat() * m_lengthOfLines;
			if (d > m_lengthOfLines - 1e-7f) d = m_lengthOfLines - 1e-7f;
			
			for (int i = 0; i < m_lines.length; ++i)
			{
				d -= (m_lines[i].length());
				if (d < 0)
				{
					return m_lines[i];
				}
			}
			assert(false);
			*/
			
			return m_lines[randRange(m_lines.length)];
		}
		
		// a random point on the GDO
		void randPoint(out vec2f pos, out float angle, out vec4f color)
		{
			GDOLine l = randLine();
			float t = randFloat();
			pos = l.getPoint(t);
			color = l.getColor(t);
			angle = l.angle();
		}
		
		void findtotalLength()
		{
			m_lengthOfLines = 0.f;
			foreach (GDOLine l; m_lines)
			{
				l.computeLength();
				m_lengthOfLines += l.length();	
			}	
		}
		
		/// bring back all XY coordinates in [-1..1]
		void normalize()
		{
			assert(m_lines.size() > 0);
			float xmin = float.max;
			float ymin = float.max;
			float xmax = float.min;
			float ymax = float.min;
			foreach (GDOLine l; m_lines)
			{
				if (l.m_a.x < xmin) xmin = l.m_a.x;
				if (l.m_a.x > xmax) xmax = l.m_a.x;
				if (l.m_a.y < ymin) ymin = l.m_a.y;
				if (l.m_a.y > ymax) ymax = l.m_a.y;
				if (l.m_b.x < xmin) xmin = l.m_b.x;
				if (l.m_b.x > xmax) xmax = l.m_b.x;
				if (l.m_b.y < ymin) ymin = l.m_b.y;
				if (l.m_b.y > ymax) ymax = l.m_b.y;
			}
			
			float xsize = (xmax - xmin);
			float ysize = (ymax - ymin);
			
			float xmid = 0.5f * (xmax + xmin);
			float ymid = 0.5f * (ymax + ymin);
			
			float f = 1.f / max(xsize, ysize);
			
			foreach (GDOLine l; m_lines)
			{				
				l.m_a.x = (l.m_a.x - xmid) * f;
				l.m_a.y = -(l.m_a.y - ymid) * f;
				l.m_b.x = (l.m_b.x - xmid) * f;
				l.m_b.y = -(l.m_b.y - ymid) * f;
			}
		
		}
	}
}

