/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 


module demo.scene20;

import demo.effect;
import util.all;
import util.geometry.all;
import gl.all;
import demo.tunnelpath;


final class Scene20 : Effect
{
	private 
	{
		ShaderProgram m_shader;
		Vector!(PaperObject) m_objects;
		
		const N_SILHOUETTES = 32;
		Silhouette[N_SILHOUETTES] m_silhouettes;
		int m_current = 0;
		int m_silcount = 0;
		
		const ORANGE_GUY = 24;
	}
	
	public
	{		
		this(vec2d interval, int priority)
		{
			super("Scene20", interval, priority);			
		}	
		
		final ShaderProgram putShader()
		{
			return m_shader;
		}
		
		void newSilhouette()
		{		
			int type = 0;			
			
			
			
			if (m_silcount == 22) type = 2;
			if (m_silcount == 19) type = 2;
			if (m_silcount == 28) type = 2;
			if (m_silcount == 20) type = 1;
			if (m_silcount == 27) type = 1;
			if (m_silcount == 16) type = 1;
				
		
			//vec4f c = hsla_to_rgba(randFloat(), 0.7f , 0.5f, 1.f);
			
			auto c = vec4f(0.f, 0.f, 0.f, 1.f); // hsla_to_rgba(randFloat(), 0.7f , 0.5f, 1.f);
			
			bool reverted = false;
			if (m_silcount == ORANGE_GUY) 
			{
				c = vec4f(1.f,0.5f,0.f,1.f); // ORANGE GUY (m_type == 3)
				type = 3;
			}
			
			m_silhouettes[m_current].m_type = type;
			m_silhouettes[m_current].m_progression = 0.f;
			m_silhouettes[m_current].m_color = c;
			m_silhouettes[m_current].m_reverted = reverted;
			
			m_current = (m_current + 1) % N_SILHOUETTES;
			m_silcount++;
		}
		
		override bool isPaperEffect() { return true; }	
		
		override void onLoad() // load resources
		{	
			m_shader = new ShaderProgram("data/shaders/put_vertex.glsl", "data/shaders/put_pixel.glsl");
			
			for (int i = 0; i < N_SILHOUETTES; ++i)
			{
				m_silhouettes[i] = new Silhouette(this, -1.f);
			}
			
			m_objects = new Vector!(PaperObject)();			
			m_objects.add(new Tour(this));
			
			for (int i = 0; i < N_SILHOUETTES; ++i)
			{
				m_objects.add(m_silhouettes[i]);
			}
			
			m_objects.add(new Escalator(this));
			m_objects.add(new EscalatorPart2(this));			
			m_objects.add(new MurG(this));
			m_objects.add(new MurD(this));
			m_timeToNextSilhouette = timePerBeat * 5.f;
		}	
			
		override void onDraw() // draw effect
		{
			demo.mainFBO.use();	
					
		    m_shader.use();
		    
		  	GL.disable( [ GL_CULL_FACE, GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH ] );     
		  	
		  	GL.enable(GL_BLEND);
		  	GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA, 
		                GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);	
		    
		/*    GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f)) * mat4f.perspective( 45, 1.f, 0.01, 1000.f);
			GL.modelviewMatrix = mat4f.lookAt(vec3f(0,0,-2.0f), vec3f(0,0,0), vec3f(0,1,0));			
		*/	
			float zoom = 1.f + 0.3f * smoothStep(timePerBeat * 4.f, timePerBeat * 6.f, localTime);
			
			auto open_brain = smoothStep(timePerBeat * 32.0f, timePerBeat * 44f, localTime);
			auto open_brain2 = smoothStep(timePerBeat * 40.0f, timePerBeat * 48f, localTime);
			auto open_brain3 = smoothStep(timePerBeat * 46.0f, timePerBeat * 48f, localTime);
			auto constant_prout = 0.2f * step(0, 48.f, localTime);
			
			zoom += 1.f * open_brain;
			zoom += 1.f * constant_prout;
			zoom += 100.f * open_brain2;
			zoom += 100.f * open_brain3;
			zoom += constant_prout;
			
			float angleZ = 1.f * TWO_PI_F * smoothStep(timePerBeat * 40.0f, timePerBeat * 48f, localTime);;
			
	/*		if (localBeat >= 6) zoom += 0.07f * demo.musicVolume;  //removed by wullon : too much*/
		
			GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f));
			auto where_is_orange_guy_head = m_silhouettes[ORANGE_GUY % N_SILHOUETTES].m_pos + vec3f(0.f,0.05f,0);
			GL.modelviewMatrix = mat4f.scale(vec3f(zoom)) * mat4f.rotateZ(angleZ) * mat4f.translate(open_brain * (-where_is_orange_guy_head )) ;
			
			foreach(PaperObject o; m_objects)
				o.draw();
		}
		
		float m_timeToNextSilhouette;
		
		void onMove(double dt) // move effect
		{
			m_timeToNextSilhouette -= dt;
			if (m_timeToNextSilhouette < 0.f)
			{
				m_timeToNextSilhouette += timePerBeat;	
				newSilhouette();
			}
			
			
			foreach(PaperObject o; m_objects)
				o.onMove(dt, localTime);
		}
	}
}

class PaperObject // 2D like objects for this scene
{
	private
	{
		vec3f m_pos;	
		float m_angle;
		float m_size;
		Scene20 m_effect;			
	}
	
	protected
	{
		abstract void onDraw();
		abstract void onMove(float dt, float localTime);
	}
	
	public
	{
		final void draw()
		{
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();
			glTranslatef(+m_pos.x, +m_pos.y, +m_pos.z);
			glRotatef(m_angle, 0,0,1);
			glScalef(m_size, m_size, m_size);
			onDraw();	
			glPopMatrix();
		}
		
		this(Scene20 effect, vec3f pos, float angle = 0.f, float size = 1.f)
		{
			m_angle = angle;
			m_pos = pos;
			m_size = size;
			m_effect = effect;
		}
		
		final vec3f pos() { return m_pos; }
		final vec3f pos(vec3f p) { return m_pos = p; }
		final Scene20 effect() { return m_effect; }
	}
}

final class Silhouette : PaperObject
{
	private
	{
		vec3f getPos(float progression)
		{
			auto a = vec2f(-1.0f,0.2f);
			auto b = vec2f(-0.6f,0.2f);
			auto c = vec2f(0.4f,-0.6f);
			auto d = vec2f(1.0f,-0.6f);
			vec2f p;
			auto t = progression * 2.f;
			if (t < 0.4f)
			{
				p = lerp(a, b, t / 0.4f);
			} else if (t < 1.4)
			{
				p = lerp(b, c, (t-0.4f) / 1.0f);
			} else 
			{
				p = lerp(c, d, (t-1.4f) / 0.4f);
			}
			
			const LSAUT = 0.12f;
			//float saut = LSAUT  * m_effect.demo.musicVolume;
			float fbeat = effect.localTime / effect.timePerBeat;
			
			float decal = progression * 0.2f;
			float issaut = 0
			             //+ max(0.f, 1.f - 2.f * abs(8.5f - fbeat - decal)) 
			            // + max(0.f, 1.f - 2.f * abs(12.5f - fbeat - decal)) 
			             + max(0.f, 1.f - 2.f * abs(16.5f - fbeat - decal))
			             + max(0.f, 1.f - 2.f * abs(20.5f - fbeat - decal))
			             + max(0.f, 1.f - 2.f * abs(24.5f - fbeat - decal))
			             + max(0.f, 1.f - 2.f * abs(28.5f - fbeat - decal)); 
			 
			float saut = (1.f - sqr(1.f - issaut)) * LSAUT;
			
			
			p.y += 0.02f + saut;
			
			
			return vec3f(p, 0.f);
		}
		
		static ShaderProgram m_shader = null;
		static Texture2D[4] m_texs; 
	}
	
	protected
	{
		override void onMove(float dt, float localTime)
		{
			if (m_progression > -0.1f)
			{
				float realitystrikesback = smoothStep(effect.timePerBeat * 32.f, effect.timePerBeat * 36.f, effect.localTime);
				m_progression += dt * 0.15f * (1.f - realitystrikesback);
				m_pos = getPos(m_progression);
				if (m_progression > 1.f)
				{
					m_progression = -0.1f;
				}
			}
		}	
		
		override void onDraw()
		{
			if (m_progression > -0.1f)
			{
				int reverted = m_reverted ? 1 : 0;				
				vec4f color = m_color;				
				
				if (m_type != 3)
				{
				
					m_shader.set1f("amountTex0", m_type == 0 ? 1.f : 0.f);
					m_shader.set1f("amountTex1", m_type == 1 ? 1.f : 0.f);
					m_shader.set1f("amountTex2", m_type == 2 ? 1.f : 0.f);
					m_shader.set1f("amountTex3", 0.f);
					
					
										
				} else  // orange guy
				{
					auto open_brain = smoothStep(effect.timePerBeat * 38.0f, effect.timePerBeat * 42.0f, effect.localTime);
					
					
					m_shader.set1f("amountTex0", 1.f - open_brain );
					m_shader.set1f("amountTex1", 0.f);
					m_shader.set1f("amountTex2", 0.f);
					m_shader.set1f("amountTex3", open_brain);
					
					// draw a circle
					
					m_effect.putShader().use();					
					const N = 40;
					
					static vec2f[N+1] vv;
					static bool vv_inited = false;
					vec2f pos = vec2f(+0.005f,0.05f);
					
					if (!vv_inited)
					{
						vv_inited = true;
						for (int i = 0; i <= N; i++)
						{
							auto t = TWO_PI_F * i / cast(float)N;
							vv[i] = vec2f(cos(t), sin(t));
						}
					}
					
					glBegin(GL_TRIANGLES);
					glColor4f(0.f,0.f,0.f, open_brain);
					for (int i = 0; i < N; i++)
					{
						GL.vertex(pos , pos + vv[i] * 0.022f, pos + vv[i+1] * 0.022f);						
					}					
					glEnd();
					
				}
					
				
				m_shader.use();
				glBegin(GL_QUADS);				
				
				glColor4fv(color.ptr);
				glTexCoord2f(1 - reverted,1); glVertex2f(-0.1, -0.1f);
				glTexCoord2f(reverted,1); glVertex2f(+0.1, -0.1f);
				glTexCoord2f(reverted,0); glVertex2f(+0.1, +0.1f);
				glTexCoord2f(1 - reverted,0); glVertex2f(-0.1, +0.1f);
				
				glTexCoord2f(1 - reverted,1); glVertex2f(-0.1, -0.1f);
				glTexCoord2f(reverted,1); glVertex2f(+0.1, -0.1f);
				glTexCoord2f(reverted,0); glVertex2f(+0.1, -0.3f);
				glTexCoord2f(1 - reverted,0); glVertex2f(-0.1, -0.3f);
				glEnd();
			}
		}		
	}	
	
	public
	{	
		vec4f m_color;
		float m_progression; // progression sur le chemin des silhouette
		int m_type; // 0 : gars, 1 : chapeau, 2 : fille a gros seins
		bool m_reverted;
		
		this(Scene20 effect, float progression)
		{
			auto p = vec3f(-10,0,0);
			m_type = 0;
			m_size = 0.7f;
			super(effect, p);
			m_progression = progression;
			m_color = vec4f(1,1,1,1);
			m_reverted = false;
			
			if (m_shader is null)
			{
				m_shader = new ShaderProgram("data/shaders/puttexture_vertex.glsl", "data/shaders/puttexture_pixel.glsl");
				for (int i = 0; i < 4; ++i)
				{
					auto names = ["data/tex/silhouette1.png", "data/tex/silhouette2.png", "data/tex/silhouette3.png", "data/tex/silhouette4.png"];
					m_texs[i] = effect.demo.loadTexture2D(names[i],
					                                      Texture.Filter.LINEAR_MIPMAP_LINEAR,
					                                      Texture.Filter.LINEAR,
					                                      Texture.Wrap.CLAMP_TO_EDGE,
					                                      true);					
				}				
				m_shader.setSampler("tex0", m_texs[0], 0);
				m_shader.setSampler("tex1", m_texs[1], 1);
				m_shader.setSampler("tex2", m_texs[2], 2);
				m_shader.setSampler("tex3", m_texs[3], 3);
			}
		}	
				
	}	
}

final class Escalator : PaperObject
{
	protected
	{
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;
			glBegin(GL_QUADS);
			glColor4f(0.0f,0.0f,0.0f,1.f);
			glVertex2f(-0.91, 0.2f);
			glVertex2f(-0.6, 0.2f);
			glVertex2f(-0.6, -1.1f);
			glVertex2f(-0.91, -1.1f);
			
			glVertex2f(-0.6, 0.2f);
			glVertex2f(-0.6, -1.1f);
			glVertex2f(0.4, -1.1f);
			glVertex2f(0.4, -0.6f);
			
			glEnd();
		}
		
		override void onMove(float dt, float localTime)
		{
			float s = m_effect.timePerBeat * 3.f;
			vec3f start = vec3f(-3.f, 0.f, 0.f);
			vec3f stop = vec3f(0.f, 0.f, 0.f);
			float t = smoothStep(s, s + m_effect.timePerBeat, localTime);
			m_pos = lerp(start, stop, t);
		}
	}
	
	public
	{		
		this(Scene20 effect)
		{
			super(effect, vec3f(-5.f, 0.f, 0.f));
		}		
	}
}


final class EscalatorPart2 : PaperObject
{
	protected
	{
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;
			glBegin(GL_QUADS);
			glColor4f(0.0f,0.0f,0.0f,1.f);
			glVertex2f(0.4, -1.1f);
			glVertex2f(0.4, -0.6f);
			glVertex2f(0.91, -0.6f);
			glVertex2f(0.91, -1.1f);			
			glEnd();
		}
		
		override void onMove(float dt, float localTime)
		{
			float s = m_effect.timePerBeat * 3.f;
			vec3f start = vec3f(+3.f, 0.f, 0.f);
			vec3f stop = vec3f(0.f, 0.f, 0.f);
			float t = smoothStep(s, s + m_effect.timePerBeat, localTime);
			m_pos = lerp(start, stop, t);
		}
	}
	
	public
	{		
		this(Scene20 effect)
		{
			super(effect, vec3f(-5.f, 0.f, 0.f));
		}		
	}
}


final class MurG : PaperObject
{
	protected
	{
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;
			glBegin(GL_QUADS);
			glColor4f(0.0f,0.0f,0.0f,1.f);
			
			auto xmin = -ratio * 1.5f, xmax = -0.8f;
			auto ymin = -1.01f, ymax = +0.9f;
			
			glVertex2f(xmin, ymin);
			glVertex2f(xmax, ymin);
			glVertex2f(xmax, ymax);
			glVertex2f(xmin, ymax);
			
			
			
			
			auto const WINDOW_SIZE = 0.08f;
		
			float bx = xmax - WINDOW_SIZE * 2.f;
			while (bx > xmin)
			{
				float by = ymax - WINDOW_SIZE * 2.f;
				while (by > ymin)
				{
					glColor4f(1.f,1.f,1.f,1.f);			
					glVertex2f(bx, by);
					glVertex2f(bx - WINDOW_SIZE , by);
					glVertex2f(bx- WINDOW_SIZE, by- WINDOW_SIZE);
					glVertex2f(bx, by- WINDOW_SIZE);
					
					glColor4f(0.0f,0.0f,0.0f,1.f);
					auto d = WINDOW_SIZE / 12.f;
					glVertex2f(bx - WINDOW_SIZE * 0.5f + d, by);
					glVertex2f(bx - WINDOW_SIZE * 0.5f - d, by);
					glVertex2f(bx - WINDOW_SIZE * 0.5f - d, by- WINDOW_SIZE);
					glVertex2f(bx - WINDOW_SIZE * 0.5f + d, by- WINDOW_SIZE);					
					glVertex2f(bx, by - WINDOW_SIZE * 0.5f + d);
					glVertex2f(bx - WINDOW_SIZE, by - WINDOW_SIZE * 0.5f + d);
					glVertex2f(bx - WINDOW_SIZE, by - WINDOW_SIZE * 0.5f - d);
					glVertex2f(bx, by - WINDOW_SIZE * 0.5f - d);
					
					by -= WINDOW_SIZE * 4.f;
				}
				bx -= WINDOW_SIZE * 3.f;
			}
			
			
			glEnd();
			
			
		}
		
		override void onMove(float dt, float localTime)
		{
			float s = m_effect.timePerBeat * 1.f;
			vec3f start = vec3f(0.f, -3.f, 0.f);
			vec3f stop = vec3f(0.f, 0.f, 0.f);
			float t = smoothStep(s, s + m_effect.timePerBeat , localTime);
			m_pos = lerp(start, stop, t);
		}
	}
	
	public
	{		
		this(Scene20 effect)
		{
			super(effect, vec3f(0.f, -5.f, 0.f));
		}		
	}
}

final class MurD : PaperObject
{
	protected
	{
		
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;
			
			auto xmin = 0.8f, xmax = ratio * 1.5f;
			auto ymin = -1.01f, ymax = +0.45f;
			glBegin(GL_QUADS);
			glColor4f(0.0f,0.0f,0.0f,1.f);
			glVertex2f(xmin, ymin);
			glVertex2f(xmax, ymin);
			glVertex2f(xmax, ymax);
			glVertex2f(xmin, ymax);
			
			
			
			glColor4f(1.f,1.f,1.f,1.f);			
			auto const WINDOW_SIZE = 0.08f;
		
			float bx = xmin + WINDOW_SIZE * 2.f;
			while (bx < xmax)
			{
				float by = ymax - WINDOW_SIZE * 2.f;
				while (by > ymin)
				{
					glColor4f(1.f,1.f,1.f,1.f);
					glVertex2f(bx, by);
					glVertex2f(bx + WINDOW_SIZE , by);
					glVertex2f(bx + WINDOW_SIZE, by- WINDOW_SIZE);
					glVertex2f(bx, by- WINDOW_SIZE);
					
					glColor4f(0.0f,0.0f,0.0f,1.f);
					auto d = WINDOW_SIZE / 12.f;
					glVertex2f(bx + WINDOW_SIZE * 0.5f + d, by);
					glVertex2f(bx + WINDOW_SIZE * 0.5f - d, by);
					glVertex2f(bx + WINDOW_SIZE * 0.5f - d, by- WINDOW_SIZE);
					glVertex2f(bx + WINDOW_SIZE * 0.5f + d, by- WINDOW_SIZE);
					glVertex2f(bx, by - WINDOW_SIZE * 0.5f + d);
					glVertex2f(bx + WINDOW_SIZE, by - WINDOW_SIZE * 0.5f + d);
					glVertex2f(bx + WINDOW_SIZE, by - WINDOW_SIZE * 0.5f - d);
					glVertex2f(bx, by - WINDOW_SIZE * 0.5f - d);
					
					by -= WINDOW_SIZE * 4.f;
				}
				bx += WINDOW_SIZE * 3.f;
			}
			glEnd();
		}
		
		
		
		override void onMove(float dt, float localTime)
		{
			float s = m_effect.timePerBeat * 2.f;
			vec3f start = vec3f(0.f, -3.f, 0.f);
			vec3f stop = vec3f(0.f, 0.f, 0.f);
			float t = smoothStep(s, s + m_effect.timePerBeat, localTime);
			m_pos = lerp(start, stop, t);
		}
	}
	
	public
	{		
		this(Scene20 effect)
		{
			super(effect, vec3f(0.f, -5.f, 0.f));
		}		
	}
}


final class Tour : PaperObject
{
	protected
	{
		vec4f color;
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;
			
			for (int i = 0; i < 2; ++i)
			{
				glBegin(GL_QUAD_STRIP);
				if (i == 0) glColor4f(color.x, color.y, color.z, color.w * 0.5f);
				else glColor4fv(color.ptr); 
				
				auto ymin = -1.01f, ymax = +0.9f;
				
				float y = ymin;
				const float FLOOR = 0.01f;
				float width = i == 0 ? 0.11f : 0.1f;
				int f = 0;
				while (y < ymax)
				{
					glVertex2f(-width, y);
					glVertex2f(+width, y);				
					y += FLOOR;
					width *= 0.99f;	
					if (f == 170) width = 0.051f;
					if (f == 180) width *= 0.2f;				
					f++;
				}
				glEnd();
			}
			
			
			
		}
		
		override void onMove(float dt, float localTime)
		{
			float s = 0;
			vec4f start = vec4f(0.7f, 0.7f, 0.7f, 0.f);
			vec4f stop = vec4f(0.7f, 0.7f, 0.7f, 1.f);
			float t = smoothStep(s, s + m_effect.timePerBeat * 8.f, localTime);
			m_pos = vec3f(0.f, 0.f, 0.f);
			color = lerp(start, stop, t);
		}
	}
	
	public
	{		
		this(Scene20 effect)
		{
			super(effect, vec3f(0.f, 0.f, 0.f));			
		}		
	}
}