/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.scene30;

import demo.effect;
import util.all;
import util.geometry.all;
import gl.all;
import demo.tunnelpath;


final class Scene30 : Effect
{
	private 
	{
		ShaderProgram m_shader;
		Vector!(PaperObject) m_objects;
		
		const N_SILHOUETTES = 32;
		Silhouette[N_SILHOUETTES] m_silhouettes;
		
		int m_current  = 0;
		int m_currentObject  = 0;
		int m_silcount = 0;
		
		int m_marcurrent  = 0;
		int m_marcount = 0;
		
		const ORANGE_GUY = 24;
		
		
		
		
	}
	
	public
	{		
		this(vec2d interval, int priority)
		{
			super("Scene30", interval, priority);			
		}	
		
		final ShaderProgram putShader()
		{
			return m_shader;
		}
		
		void newSilhouette()
		{		
			int type = 0;			
			
			if (m_silcount == 12) type = 4; // the orange guy
			
			m_silhouettes[m_current].m_type = type;
			m_silhouettes[m_current].m_progression = 0.f;
			m_silhouettes[m_current].m_color = type == 4 ? vec4f(1.f,0.5f,0.f,1.f) : vec4f(0.f,0.f,0.f,1.f);
			
			m_current = (m_current + 1) % N_SILHOUETTES;
			m_silhouettes[m_current].m_index = m_silcount;
			m_silcount++;
			
		}
	
		override bool isPaperEffect() { return true; }	
		
		override void onLoad() // load resources
		{	
			m_shader = new ShaderProgram("data/shaders/put_vertex.glsl", "data/shaders/put_pixel.glsl");
						
			for (int i = 0; i < N_SILHOUETTES; ++i)
			{
				m_silhouettes[i] = new Silhouette(this, -1.f);
			}
			
			m_objects = new Vector!(PaperObject)();
		
			
			for (int i = 0; i < N_SILHOUETTES; ++i)
			{
				m_objects.add(m_silhouettes[i]);
			}
	
			m_objects.add(new Walls(this));
			
			m_timeToNextSilhouette = timePerBeat * 2.f;
			
		
		}	
			
		override void onDraw() // draw effect
		{
			demo.mainFBO.use();	
					
		    m_shader.use();
			
			//GL.clear();
		    
		  	GL.disable( [ GL_CULL_FACE, GL_DEPTH_TEST, GL_ALPHA_TEST, GL_LINE_SMOOTH, GL_POLYGON_SMOOTH, GL_POINT_SMOOTH ] );    
			
		  	glClearColor(1.0,1.0,1.0,1.0);
		  	glClear(GL_COLOR_BUFFER_BIT);
			
		  	GL.enable(GL_BLEND);
		  	GL.setBlend(GL.BlendMode.ADD, GL.BlendFactor.SRC_ALPHA, GL.BlendFactor.ONE_MINUS_SRC_ALPHA, 
		                GL.BlendMode.MAX, GL.BlendFactor.ONE, GL.BlendFactor.ONE);	
		    
		
			GL.projectionMatrix = mat4f.scale(vec3f(1.f / demo.ratio, 1.f, 1.f));
			GL.modelviewMatrix = mat4f.IDENTITY;
			
			foreach(PaperObject o; m_objects)
				o.draw();
		}
		
		float m_timeToNextSilhouette;
		
		void onMove(double dt) // move effect
		{
			m_timeToNextSilhouette -= dt;
			if (m_timeToNextSilhouette < 0.f)
			{
				m_timeToNextSilhouette += timePerBeat;	
				newSilhouette();
			}			
			
			foreach(PaperObject o; m_objects)
				o.onMove(dt, localTime);
		}
	}
}

class PaperObject // 2D like objects for this scene
{
	private
	{
		vec3f m_pos;	
		float m_angle;
		float m_size;
		Scene30 m_effect;			
	}
	
	protected
	{
		abstract void onDraw();
		abstract void onMove(float dt, float localTime);
	}
	
	public
	{
		final void draw()
		{
			glMatrixMode(GL_MODELVIEW);
			glPushMatrix();
			glTranslatef(+m_pos.x, +m_pos.y, +m_pos.z);
			glRotatef(m_angle, 0,0,1);
			glScalef(m_size, m_size, m_size);
			onDraw();	
			glPopMatrix();
		}
		
		this(Scene30 effect, vec3f pos, float angle = 0.f, float size = 1.f)
		{
			m_angle = angle;
			m_pos = pos;
			m_size = size;
			m_effect = effect;
		}
		
		final vec3f pos() { return m_pos; }
		final vec3f pos(vec3f p) { return m_pos = p; }
		final Scene30 effect() { return m_effect; }
	}
}

final class Silhouette : PaperObject
{
	private
	{
		vec3f getPos(float progression)
		{
			
			if (progression < 0.5f)
			{
				return vec3f(lerp(-4.f / 3.f - 0.1f, 4.f / 3.f + 0.1f, progression * 2.f), 0.4f, 0.f);	
			} else
			{
				//float  y = ((m_progression > 0.5f) && (m_type != 4)) ? -0.7 : -0.6;
				float y = -0.6f;
				return vec3f(lerp(+4.f / 3.f + 0.1f, -4.f / 3.f - 0.1f, (progression-0.5f) * 2.f), y, 0.f);		
			}
		}
		
		static ShaderProgram m_shader = null;
		static Texture2D[4] m_texs; 
	}
	
	protected
	{
		override void onMove(float dt, float localTime)
		{
			if (m_progression > -0.1f)
			{
				bool under_half = m_progression < 0.5f;
				if ((m_progression > 0.5f))
				{
					m_progression += dt * 0.10f;
				} else
				{				
					m_progression += dt * (0.10f + 0.10f * sin(m_index + effect.localTime * 2.0f * PI_F / effect.timePerBeat));
				}
				m_pos = getPos(m_progression);
				m_reverted = m_progression > 0.5f;
				if (under_half && (m_progression > 0.5f) && (m_type != 4))
				{
					m_type = 1;
				}
				
				
				if (m_progression > 1.f)
				{
					m_progression = -0.1f;
				}
			}
		}	
		
		override void onDraw()
		{
			if (m_progression > -0.1f)
			{
				int reverted = m_reverted ? 1 : 0;				
				vec4f color = m_color;				
				
				m_shader.set1f("amountTex0", ((m_type == 0) || (m_type == 4)) ? 1.f : 0.f);
				m_shader.set1f("amountTex1", m_type == 1 ? 1.f : 0.f);
				m_shader.set1f("amountTex2", m_type == 2 ? 1.f : 0.f);
				m_shader.set1f("amountTex3", m_type == 3 ? 1.f : 0.f);
				
				m_shader.use();
				glBegin(GL_QUADS);				
				glColor4fv(color.ptr);
				glTexCoord2f(1 - reverted,1); glVertex2f(-0.2, -0.2f);
				glTexCoord2f(reverted,1); glVertex2f(+0.2, -0.2f);
				glTexCoord2f(reverted,0); glVertex2f(+0.2, +0.2f);
				glTexCoord2f(1 - reverted,0); glVertex2f(-0.2, +0.2f);			
				glEnd();
			}
		}		
	}	
	
	public
	{	
		int m_index;
		vec4f m_color;
		float m_progression; // progression sur le chemin des silhouette
		int m_type; // 0 : gars, 1 : chapeau, 2 : fille a gros seins
		bool m_reverted;
		
		this(Scene30 effect, float progression)
		{
			auto p = vec3f(-10,0,0);
			m_type = 0;
			m_size = 0.7f;
			super(effect, p);
			m_progression = progression;
			//m_color = color;
			m_color = vec4f(0,0,0,1);
			m_reverted = false;
			
			if (m_shader is null)
			{
				m_shader = new ShaderProgram("data/shaders/puttexture_vertex.glsl", "data/shaders/puttexture_pixel.glsl");
				for (int i = 0; i < 2; ++i)
				{
					auto names = ["data/tex/silhouette1.png", 
					              "data/tex/chocopaf.png" ];
					m_texs[i] = effect.demo.loadTexture2D(names[i],
					                                      Texture.Filter.LINEAR_MIPMAP_LINEAR,
					                                      Texture.Filter.LINEAR,
					                                      Texture.Wrap.CLAMP_TO_EDGE,
					                                      true);					
				}				
				m_shader.setSampler("tex0", m_texs[0], 0);
				m_shader.setSampler("tex1", m_texs[1], 1);
				m_shader.setSampler("tex2", m_texs[0], 2);
				m_shader.setSampler("tex3", m_texs[1], 3);
			}
		}	
				
	}	
}

final class Walls : PaperObject
{
	protected
	{
		override void onDraw()
		{
			m_effect.putShader().use();
			auto ratio = m_effect.ratio;			
			
			glBegin(GL_QUADS);
			glColor4f(0.0f,0.0f,0.0f,1.f);			
			
			{
				auto t1 = smoothStep(0.f, 1.f * effect.timePerBeat, effect.localTime);	
				
				auto a = vec2f(-4.f / 3.f, 1) + vec2f(-0.9,+0.9);
				auto b = vec2f(-4.f / 3.f, -1) + vec2f(-0.9,-0.9);
				auto c = vec2f(+4.f / 3.f, -1) + vec2f(+0.9,-0.9);
				auto d = vec2f(+4.f / 3.f, 1) + vec2f(+0.9,+0.9);	
				
				auto e = vec2f(-4.f / 3.f, 1)  + vec2f(+0.2,-0.2) * t1;
				auto f = vec2f(-4.f / 3.f, -1) + vec2f(+0.2,+0.2) * t1;
				auto g = vec2f(+4.f / 3.f, -1)  + vec2f(-0.2,+0.2) * t1;
				auto h = vec2f(+4.f / 3.f, 1)  + vec2f(-0.2,-0.2) * t1;
				
				GL.vertex(a,e,f,b,b,f,g,c,c,g,h,d,d,h,e,a);
				
				auto t2 = smoothStep(1.f * effect.timePerBeat, 2.f * effect.timePerBeat, effect.localTime);	
				auto s = vec2f(-4.f / 3.f + t2 * 8.f / 3.f, 0.2f);
				auto t = vec2f(-4.f / 3.f + t2 * 8.f / 3.f, -0.2f);
				auto u = vec2f(-4.f / 3.f, -0.2f);
				auto v = vec2f(-4.f / 3.f, 0.2f);		
				GL.vertex(s, t, u, v);
			}			
			glEnd();
		}
		
		override void onMove(float dt, float localTime)
		{
			m_pos = vec3f(0,0,0);
		}
	}
	
	public
	{		
		this(Scene30 effect)
		{
			super(effect, vec3f(0.f, 0.f, 0.f));
		}		
	}
}