/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module demo.tunnelpath;

import util.all;
import util.geometry.all;

// code from Turbolol Destruction Trooper, the most difficult game to explain ever
// translated to D by #ponce



class InterestingTunnelPath
{
	private
	{
	
		class TunnelSlice /// represents a logical slice in the tunnel 
		{
			vec3f position; // position of the center of the slice
			vec3f up; // direction of the up vector at this point (normalized)
			vec3f dir; // direction of the path at this point (normalized)
			float radius; // radius of the tunnel at this point
			float length;
			
			this(vec3f _pos, vec3f _up, vec3f _dir, float _radius, float _length)
			{
			    position = _pos;
				up = _up.normalized();
				dir = _dir.normalized();
				radius = _radius;
				length = _length;
			}
		}
		    
	    Vector!(TunnelSlice) m_slices;
	    float m_length;
	    
	      // allows to index m_slice without overflow
	    TunnelSlice getSlice(int i) 
	    {
	        int nslices = m_slices.length();
	        if (i < 0) i = 0;
	        else if (i >= nslices) i = nslices - 1;
	        return m_slices[i];
	    }
	
	    // give a valid index and float point part for a given progression
	    void decompose(float s, out int ipart, out float fpart)
	    {
		    // loop when becoming larger than length
		  //  while (s > m_length)
		    //    s -= m_length;
		
		    if (s < 0) {
		        ipart = 0;
		        fpart = 0;
		        return;
		    } else if (s >= m_slices.length - 1) 
		    {
			 	   ipart = m_slices.length - 2;
			 	   fpart = 1.f;
		    } else
		    {
			 	ipart = trunc(s); 
			 	fpart = s - ipart;
		    }   
		    
	    }	    
	    
		T hermite(T)(float frac_pos, T xm1, T x0, T x1, T x2)
		{
		   final T c = (x1 - xm1) * 0.5f;
		   final T v = x0 - x1;
		   final T w = c + v;
		   final T a = w + v + (x2 - x0) * 0.5f;
		   final T b_neg = w + a;		
		   return ((((a * frac_pos) - b_neg) * frac_pos + c) * frac_pos + x0);
		}
		
		float m_distanceBetweenSlices;
		int m_slicesNumber;
    }
    
    public
    {

    	
    	this(int nsclices, float distance, vec3f initialPosition, vec3f initialUp, vec3f initialDir)
    	{
	    	m_slicesNumber = nsclices;
	    	m_distanceBetweenSlices = distance;
	    	m_length = 0;
	    	m_slices = new Vector!(TunnelSlice)();
	    	
			vec3f pos = initialPosition;
    		vec3f up  = initialUp.normalized();
    		vec3f dir = initialDir.normalized();   	
    		vec3f right = dir.cross(up);    		
	
			float upRot = 0.f;  // "LACET"
			float dirRot = 0.f;	// "ROULIS"
			float rightRot = 0.f; // "TANGAGE"	
			float vupRot = 0.f;  // derivee du "LACET"
			float vdirRot = 0.f;	// derivee du "ROULIS"
			float vrightRot = 0.f; // derivee du "TANGAGE"
			
			const float VISCOSITY = 0.4f;		
			const float VISCOSITY2 = 0.5f;			
			const float DAMPING = 0.3f;

			const float TARGET_RADIUS = 3.f; // the radius to be attracted to    	
			
			float radius = TARGET_RADIUS;
			float vradius = 0.f; // derivee du radius
		
			// the principle : we apply small Euler angles and then renormalize
			for (int i = 0; i < m_slicesNumber; i++)
		    {
			    TunnelSlice slice = new TunnelSlice(pos, up, dir, radius, m_distanceBetweenSlices);
				m_slices.add( slice );
				
				// compute next slice
				auto const upNoise = 0.10f;
				auto const dirNoise = 0.40f;
				auto const rightNoise = 0.35f;
		
				vupRot = vupRot * VISCOSITY + upNoise * randGaussf() - upRot * DAMPING;
				vdirRot = vdirRot * VISCOSITY + dirNoise * randGaussf() - dirRot * DAMPING;		
				vrightRot = vrightRot * VISCOSITY + rightNoise * (randGaussf()) - rightRot * DAMPING;				
				
				vradius = vradius * VISCOSITY + 0.4f * (randGaussf()) + (TARGET_RADIUS - radius) * DAMPING;
				
				upRot = upRot * VISCOSITY2 + vupRot;
				dirRot = dirRot * VISCOSITY2 + vdirRot;
				rightRot = rightRot * VISCOSITY2 + vrightRot;				
				radius = radius + vradius;
				
				const float ROT_FACTOR = 0.9f;
				
				// ROULIS
				right = right * cos(ROT_FACTOR * dirRot) - up * sin(ROT_FACTOR * dirRot);
				up    = right * sin(ROT_FACTOR * dirRot) + up * cos(ROT_FACTOR * dirRot);
				right.normalize();
				up.normalize();
				
				// TANGAGE
				dir = dir * cos(ROT_FACTOR * rightRot) + up * sin(ROT_FACTOR * rightRot);
				up  = dir * -sin(ROT_FACTOR * rightRot) + up * cos(ROT_FACTOR * rightRot);
				dir.normalize();	
				up.normalize();
				
				// LACET
				right = right * cos(ROT_FACTOR * upRot) - dir * sin(ROT_FACTOR * upRot);
				dir   = right * sin(ROT_FACTOR * upRot) + dir * cos(ROT_FACTOR * upRot);						
		
				right.normalize();
				dir.normalize();
				up.normalize();
				
		        pos += dir * m_distanceBetweenSlices;
		        m_length += m_distanceBetweenSlices;
		    }
    	}
    	
    	vec3f getPos(float progression)
    	{	    		
		    float fpart;
		    int ipart;          
		    decompose(progression, ipart, fpart);		
		    return hermite(fpart, getSlice(ipart-1).position, getSlice(ipart).position, 
		                          getSlice(ipart+1).position, getSlice(ipart+2).position);	    	
    	}
    	
    	vec3f getDir(float progression)
    	{	    		
		    float fpart;
		    int ipart;          
		    decompose(progression, ipart, fpart);		
		    return hermite(fpart, getSlice(ipart-1).dir, getSlice(ipart).dir, 
		                          getSlice(ipart+1).dir, getSlice(ipart+2).dir);	    	
    	}
    	
    	vec3f getUp(float progression)
    	{	    		
		    float fpart;
		    int ipart;          
		    decompose(progression, ipart, fpart);		
		    return hermite(fpart, getSlice(ipart-1).up, getSlice(ipart).up, 
		                          getSlice(ipart+1).up, getSlice(ipart+2).up);	    	
    	}
    	
    	vec3f getRight(float progression)
    	{	    		
		    vec3f dir = getDir(progression);
    		vec3f up = getUp(progression);  
    		return dir.cross(up);
    	}
    	
    	float getRadius(float progression)
    	{	    		
		    float fpart;
		    int ipart;          
		    decompose(progression, ipart, fpart);
    		return hermite(fpart, getSlice(ipart-1).radius, getSlice(ipart).radius, 
                                  getSlice(ipart+1).radius, getSlice(ipart+2).radius);
    	}
    	
    	int nSlices()
    	{
	    	return m_slicesNumber;	
    	}
	}
}