/*     DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
                    Version 2, December 2004

 Copyright (C) 2009 Adinpsz contributors
  7 chemin Villebois 38100 Grenoble
 Everyone is permitted to copy and distribute verbatim or modified
 copies of this license document, and changing it is allowed as long
 as the name is changed.

            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION

  0. You just DO WHAT THE FUCK YOU WANT TO. */
/* This program is free software. It comes without any warranty, to
 * the extent permitted by applicable law. You can redistribute it
 * and/or modify it under the terms of the Do What The Fuck You Want
 * To Public License, Version 2, as published by Sam Hocevar. See
 * http://sam.zoy.org/wtfpl/COPYING for more details. */ 
module gl.texture;

private import std.string;
private import derelict.opengl.gl;
private import derelict.opengl.glu;
private import derelict.opengl.extension.arb.texture_float;
private import derelict.opengl.extension.arb.multitexture;
private import derelict.opengl.extension.arb.pixel_buffer_object;
private import derelict.opengl.extension.ext.framebuffer_object;

private import gl.buffer;
private import gl.state;


class Texture {	
	
	public 
	{		
		enum Wrap 
		{ 
			CLAMP, 
			CLAMP_TO_EDGE, 
			REPEAT, 
			CLAMP_TO_BORDER, 
			MIRRORED_REPEAT 
		}		
		
		// internal fornat of texture
		enum IFormat 
		{ 
			I8,
			I16F,
			I32F,
			RGB8,
			RGB16F,
			RGB32F,
			RGBA8,
			RGBA16F,
			RGBA32F
		}
    	

		// format of data used to write into texture
		     
		enum Format 
		{
			INTENSITY,
			RGB,
			RGBA,
			BGR,
			BGRA,
			DEPTH
		}
		
		enum Type 
		{
			UBYTE,
			BYTE,
			USHORT,
			SHORT,
			UINT,
			INT,
			FLOAT
		}
		
		enum Filter 
		{ 
			NEAREST, 
			LINEAR, 
			NEAREST_MIPMAP_NEAREST, 
			NEAREST_MIPMAP_LINEAR, 
			LINEAR_MIPMAP_NEAREST, 
			LINEAR_MIPMAP_LINEAR 
		}
			
	}
	
	private {
		
		static const GLint[IFormat.max + 1] IFormat_toGL = 		
		[
	//		GL_INTENSITY, 
	//		GL_INTENSITY16F_ARB, 
	//		GL_INTENSITY32F_ARB, 	
			GL_LUMINANCE,
			GL_LUMINANCE16F_ARB, 
			GL_LUMINANCE32F_ARB, 
			GL_RGB,
			GL_RGB16F_ARB,
			GL_RGB32F_ARB,
			GL_RGBA,
			GL_RGBA16F_ARB,
			GL_RGBA32F_ARB
		];	
		
		static const GLint[Format.max + 1] Format_toGL = 		
		[
			GL_LUMINANCE,
			GL_RGB,
			GL_RGBA,
			GL_BGR,
			GL_BGRA,
			GL_DEPTH_COMPONENT
		];	
		
		static const GLint[Type.max + 1] Type_toGL = 		
		[
			GL_UNSIGNED_BYTE,
			GL_BYTE,
			GL_UNSIGNED_SHORT,
			GL_SHORT,
			GL_UNSIGNED_INT,
			GL_INT,
			GL_FLOAT
		];	
		
		static const GLint[Wrap.max + 1] Wrap_toGL = 
		[ 
			GL_CLAMP, 
			GL_CLAMP_TO_EDGE, 
			GL_REPEAT, 
			GL_CLAMP_TO_BORDER, 
			GL_MIRRORED_REPEAT 
		];
		
		static const GLint[Filter.max + 1] Filter_toGL = 
		[ 
			GL_NEAREST, 
			GL_LINEAR, 
			GL_NEAREST_MIPMAP_NEAREST, 
			GL_NEAREST_MIPMAP_LINEAR, 
			GL_LINEAR_MIPMAP_NEAREST, 
			GL_LINEAR_MIPMAP_LINEAR 
		];
		
		GLuint m_handle;
		GLint m_target;
		GLint m_width, m_height, m_depth;
		IFormat m_iFormat;
		
		final void bind() 
		{		
			glBindTexture(m_target, m_handle);				
		}
		
		final int getParam(GLenum paramName) 
		{
			
			int res;
			bind();
			glGetTexParameteriv(m_target, paramName, &res);
			return res;
			
		}		
		
		final int getLevelParam(GLenum paramName, int level)
		{			
			int res;
			bind();
			glGetTexLevelParameteriv(m_target, level, paramName, &res);
			return res;			
		}
			
	}	
		
	
	public {
		
		
		this(GLint target, IFormat iFormat) 
		{
			this.m_target = target;
			this.m_iFormat = iFormat;
			glGenTextures(1, &m_handle);
			assert(GL.check());		
			minFilter = Filter.NEAREST;	
			magFilter = Filter.NEAREST;
			wrapS = Wrap.CLAMP_TO_EDGE;
			wrapT = Wrap.CLAMP_TO_EDGE;	
			m_width = m_height = m_depth = 0;		
		}		
		
		~this() 
		{		
			glDeleteTextures( 1, &m_handle);
			assert(GL.check());				
		}
		
		
		final void use(int unit) 
		{		
			
			glActiveTexture(GL_TEXTURE0 + unit);
			glEnable(m_target);				
			assert(GL.check());
			bind();
		}
		
		
		final int width()
		{
			return m_width;
		}
		
		final int height()
		{
			return m_height;
		}
	
		final int depth()
		{
			return m_depth;
		}
				
		
		final Wrap wrapS(Wrap w) 
		{		
			bind();
			glTexParameteri(m_target, GL_TEXTURE_WRAP_S, Wrap_toGL[w]);
			assert(GL.check());
			return w;			
		}
		
		final Wrap wrapT(Wrap w) 
		{
		
			bind();
			glTexParameteri(m_target, GL_TEXTURE_WRAP_T, Wrap_toGL[w]);
			assert(GL.check());
			return w;			
		}
		
		final Wrap wrapR(Wrap w) 
		{
		
			bind();
			glTexParameteri(m_target, GL_TEXTURE_WRAP_R, Wrap_toGL[w]);
			assert(GL.check());
			return w;
			
		}
		
		final Filter minFilter(Filter f) 
		{
			
			bind();
			glTexParameteri(m_target, GL_TEXTURE_MIN_FILTER, Filter_toGL[f]);
			assert(GL.check());
			return f;			
		}
		
		final Filter magFilter(Filter f) 
		{
			
			bind();
			assert( (f == Filter.NEAREST) || (f == Filter.LINEAR) );
			glTexParameteri(m_target, GL_TEXTURE_MAG_FILTER, Filter_toGL[f]);
			assert(GL.check());
			return f;
			
		}
		
		GLint handle() 
		{		
			return m_handle;				
		}
		
		GLuint target() 
		{
		
			return m_target;			
				
		}
		
		void getImage(int level, Format format, Type type, void * pixels) 
		{
		
			bind();
			glGetTexImage(m_target, level, Format_toGL[format], Type_toGL[type], pixels);
			assert(GL.check());
				
		}
		
		static final int maxTextureUnit() 
		{
			
			GLint ntu;
			glGetIntegerv(GL_MAX_TEXTURE_UNITS, &ntu);			
			assert(GL.check());
			return ntu;
			
		}
		
		final void generateMipmaps()
		{
			bind();
			glGenerateMipmapEXT(m_target);
			assert(GL.check());		
		}
		
		final int baseLevel(int base)
		{
			bind();
			glTexParameteri(m_target, GL_TEXTURE_BASE_LEVEL, base);	
			assert(GL.check());	
			return base;
		}
		
		final int maxLevel(int level)
		{
			bind();
			glTexParameteri(m_target, GL_TEXTURE_MAX_LEVEL, level);	
			assert(GL.check());	
			return level;
		}
	}
}

class Texture1D : Texture 
{	
	public 
	{		
		this(IFormat iFormat) 
		{			
			super(GL_TEXTURE_1D,iFormat);					
		}
		
		void setImage(int level, int width, Format format, Type type, void * data) 
		{			
			bind();
			
			glTexImage1D(m_target, 
			             level, 
			             IFormat_toGL[m_iFormat], 
			             width, 
			             0, 
			             Format_toGL[format], 
			             Type_toGL[type], 
			             data);
			assert(GL.check());		
			m_width = width;	
			m_height = 1;
			m_depth = 1;
		}
		
		void setSubImage(int level, int x, int width, Format format, Type type, void * data) 
		{
			bind();
			glTexSubImage1D(m_target, 
			             level, 
			             x,
			             width, 
			             Format_toGL[format], 
			             Type_toGL[type], 
			             data);
			assert(GL.check());					
		}
		
	}
	
}

class Texture2D : Texture 
{	
	public 
	{		
		this(IFormat iFormat) 
		{			
			super(GL_TEXTURE_2D,iFormat);			
		}
		
		void setImage(int level, int width, int height, Format format, Type type, void * data, bool mipmapped) 
		{			
			bind();
			
			
			if ( mipmapped )
			{
				gluBuild2DMipmaps(m_target, 
				                  IFormat_toGL[m_iFormat], 
				                  width,
				                  height,
				                  Format_toGL[format],
				                  Type_toGL[type],
				                  data);
			} else
			{
				if ((this.width == width) && (this.height == height))
				{
					glTexSubImage2D(m_target,
					                0,
					                0,
					                0,
					                width,
					                height,
					                Format_toGL[format], 
					                Type_toGL[type], 
					                data);
				} else
				{
					glTexImage2D(m_target, 
					             level, 
					             IFormat_toGL[m_iFormat], 
					             width, 
					             height,
					             0, 
					             Format_toGL[format], 
					             Type_toGL[type], 
					             data);
				}
			}
			assert(GL.check());
			m_width = width;	
			m_height = height;
			m_depth = 1;			
			
		}
	
		
	
		
				
		void setSubImage(int level, int x, int y, int width, int height, Format format, Type type, void * data) {
		
			bind();
			glTexSubImage2D(m_target, 
			             level, 
			             x,
			             y,
			             width, 
			             height,
			             Format_toGL[format], 
			             Type_toGL[type], 
			             data);
			assert(GL.check());
		
		}	
	}	
}

class Texture3D : Texture 
{	
	public 
	{		
		
		this(IFormat iFormat) 
		{			
			super(GL_TEXTURE_3D,iFormat);
			
		}	
		
		void setImage(int level, int width, int height, int depth, Format format, Type type, void * data) {
			
			bind();
			glTexImage3D(m_target, 
			             level, 
			             IFormat_toGL[m_iFormat], 
			             width, 
			             height,
			             depth,
			             0, 
			             Format_toGL[format], 
			             Type_toGL[type], 
			             data);
			assert(GL.check());
			m_width = width;	
			m_height = height;
			m_depth = depth;			
		}
		
		
		void setSubImage(int level, int x, int y, int z, int width, int height, int depth, Format format, Type type, void * data) {
		
			bind();
			glTexSubImage3D(m_target, 
			             level, 
			             x,
			             y,
			             z,
			             width, 
			             height,
			             depth,
			             Format_toGL[format], 
			             Type_toGL[type], 
			             data);			
			assert(GL.check());
		
		}
		
	
		
	}
	
}
